from arepo.utils.db import populate

from sqlalchemy import create_engine
from sqlalchemy.engine import Engine
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy_utils import database_exists, create_database
from arepo.base import Base


class DatabaseConnection:
    def __init__(self, uri: str, engine: Engine = None):
        self.uri = uri if engine is None else engine.url
        self._engine = create_engine(self.uri) if engine is None else engine
        self._session_maker = sessionmaker(bind=self._engine)
        Base.metadata.bind = self._engine

        # db_session = scoped_session(sessionmaker(autocommit=False, autoflush=False, bind=engine))
        # Base.query = db_session.query_property()

    @staticmethod
    def init(uri: str):
        if not database_exists(uri):
            create_database(url=uri, encoding='utf8')

        engine = create_engine(uri)
        # Create tables
        Base.metadata.create_all(engine)
        populate(engine)

        return engine

    def get_session(self, scoped: bool = False):
        if scoped:
            db_session = scoped_session(sessionmaker(autocommit=False, autoflush=False, bind=self._engine))
            Base.query = db_session.query_property()
            return db_session
        else:
            return self._session_maker()


def get_in_memory_database() -> Engine:
    """
        This function initializes an in memory database and returns its engine.
    :return:
    """

    db_uri = "sqlite:///:memory:"
    engine = DatabaseConnection.init(db_uri)
    print('Database initialized.')
    print('engine:', engine)

    return engine



from os import environ
from arepo.db import DatabaseConnection
from arepo.models.vcs.core.repository import RepositoryAssociationModel
from arepo.models.vcs.core.commit import CommitAssociationModel

from sqlalchemy import func
from sqlalchemy.orm import aliased


def get_non_overlap_vulnerability_ids(session, model, source_1, source_2):
    alias_1 = aliased(model)
    alias_2 = aliased(model)

    # Query to find vulnerabilities in source_1 but not in source_2
    return session.query(
        alias_1.vulnerability_id
    ).distinct().outerjoin(
        alias_2,
        (alias_1.vulnerability_id == alias_2.vulnerability_id) &
        (alias_2.source_id == source_2)
    ).filter(
        alias_1.source_id == source_1,
        alias_2.vulnerability_id == None  # Where there's no match in source_2
    ).all()


db = DatabaseConnection(environ['SQLALCHEMY_DATABASE_URI'])
session = db.get_session(scoped=True)

print(
    "Total CVEs with repositories:",
    session.query(RepositoryAssociationModel.vulnerability_id).distinct().count()
)

print(
    "Total CVEs with commits:",
    session.query(CommitAssociationModel.vulnerability_id).distinct().count()
)

print("Repository Duplicates:",
      len(session.query(RepositoryAssociationModel.vulnerability_id).
          group_by(RepositoryAssociationModel.vulnerability_id).
          having(func.count(RepositoryAssociationModel.repository_id) > 1).
          all()
          )
      )

print("Commit Duplicates:",
      len(session.query(CommitAssociationModel.vulnerability_id).
          group_by(CommitAssociationModel.vulnerability_id).
          having(func.count(CommitAssociationModel.commit_id) > 1).
          all()
          )
      )


print("CVEs with repositories in NVD and not in OSV:",
      len(
          get_non_overlap_vulnerability_ids(
              session,
              model=RepositoryAssociationModel,
              source_1='nvd_id',
              source_2='osv_id'
          )
      )
      )

print("CVEs with repositories in OSV and not in NVD:",
      len(
          get_non_overlap_vulnerability_ids(
              session,
              model=RepositoryAssociationModel,
              source_1='osv_id',
              source_2='nvd_id'
          )
      )
      )

print("CVEs with commits in NVD and not in OSV:",
      len(
          get_non_overlap_vulnerability_ids(
              session,
              model=CommitAssociationModel,
              source_1='nvd_id',
              source_2='osv_id'
          )
      )
      )

print("CVEs with commits in OSV and not in NVD:",
      len(
          get_non_overlap_vulnerability_ids(
              session,
              model=CommitAssociationModel,
              source_1='osv_id',
              source_2='nvd_id'
          )
      )
      )

session.close()



import pytest

from arepo.models.source import SourceModel

source_data = [
    {
        "id": "snyk-source",
        "name": "Snyk",
        "email": "report@snyk.io"
    },
    {
        "id": "nvd-source",
        "name": "NVD",
        "email": "nvd@nist.gov"
    }
]


@pytest.mark.dependency(name="test_insert_sources")
def test_insert_sources(database_session):
    from sqlalchemy.exc import IntegrityError
    for source in source_data:
        try:
            source_model = SourceModel(
                id=source["id"],
                name=source["name"],
                email=source["email"]
            )
            database_session.add(source_model)
            database_session.commit()
        except IntegrityError:
            database_session.rollback()
            pytest.fail("IntegrityError occurred while inserting source")


@pytest.mark.dependency(depends=["test_insert_sources"])
def test_get_inserted_sources(database_session):
    # Query the database to retrieve the inserted sources
    retrieved_sources = database_session.query(SourceModel).all()

    # Check if the retrieved sources exist
    assert len(retrieved_sources) == len(source_data)

    for source in retrieved_sources:
        assert source.name in [s["name"] for s in source_data]
        assert source.email in [s["email"] for s in source_data]



import pytest

from arepo.models.vcs.diff import DiffBlockModel, ChangeModel


diff_data = {
    'diff_blocks': [
        {
            'id': 'test-diff-block-id',
            'order': 2,
            'a_path': 'src/tagify.js',
            'changes': [
                {
                    'id': 'test-change-id-1',
                    'line': 104,
                    'start_col': 9,
                    'end_col': 82,
                    'type': 'deletion',
                    'content': '        _s.placeholder = input.getAttribute(\'placeholder\') || _s.placeholder || ""'
                },
                {
                    'id': 'test-change-id-2',
                    'line': 104,
                    'start_col': 9,
                    'end_col': 94,
                    'type': 'addition',
                    'content': '        _s.placeholder = escapeHTML(input.getAttribute(\'placeholder\') || _s.placeholder || "")',
                }
            ],
            'commit_file_id': 'test-commit-file-id'
        }
    ]
}


@pytest.mark.dependency(name="insert_diff_block")
def test_insert_diff_block(database_session):
    from sqlalchemy.exc import IntegrityError

    try:
        diff_block = DiffBlockModel(
            id=diff_data['diff_blocks'][0]['id'],
            order=diff_data['diff_blocks'][0]['order'],
            a_path=diff_data['diff_blocks'][0]['a_path'],
            commit_file_id=diff_data['diff_blocks'][0]['commit_file_id']
        )

        database_session.add(diff_block)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting diff block")


@pytest.mark.dependency(depends=["insert_diff_block"])
@pytest.mark.dependency(name="insert_changes")
def test_insert_changes(database_session):
    from sqlalchemy.exc import IntegrityError

    try:
        for change_data in diff_data['diff_blocks'][0]['changes']:
            change = ChangeModel(
                id=change_data['id'],
                line=change_data['line'],
                start_col=change_data['start_col'],
                end_col=change_data['end_col'],
                type=change_data['type'],
                content=change_data['content'],
                diff_block_id=diff_data['diff_blocks'][0]['id']
            )

            database_session.add(change)
            database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting changes")




from arepo.base import Base
from arepo.utils.misc import generate_id
from arepo.mixins import EntityLoaderMixin, AssociationLoaderMixin

from sqlalchemy import Column, Integer, String, Boolean, ForeignKey, ForeignKeyConstraint
from sqlalchemy.orm import relationship


class RepositoryModel(Base, EntityLoaderMixin):
    __tablename__ = "repository"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    owner = Column('owner', String, nullable=False)
    available = Column('available', Boolean, nullable=True)
    description = Column('description', String, nullable=True)
    language = Column('language', String, nullable=True)
    size = Column('size', Integer, nullable=True)
    watchers = Column('watchers', Integer, nullable=True)
    forks = Column('forks', Integer, nullable=True)
    stargazers = Column('stargazers', Integer, nullable=True)
    commits_count = Column('commits_count', Integer, nullable=True)
    commits = relationship("CommitModel", backref="repository")

    def has_commits(self, available: bool = False, kind: str = None, has_files: bool = False,
                    has_parents: bool = False):
        """
            check if repo has commits

            :param available: flag to check if the commit is available
            :param kind: kind of commit
            :param has_files: flag to check if the commit has files
            :param has_parents: flag to check if the commit has parents

            :return:
        """

        if len(self.commits) == 0:
            return False

        commits = []

        for c in self.commits:
            # TODO: check for files and parents to avoid mismatches between count and actual entries
            if available and not c.available:
                continue

            if kind and c.kind != kind:
                continue

            if has_files and len(c.files) == 0:
                continue

            if has_parents and len(c.parents) == 0:
                continue

            commits.append(c)

        return len(commits) > 0

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the URL.
        """
        super().__init__(**kwargs)
        assert self.name is not None, "name must be provided."
        assert self.owner is not None, "owner must be provided."

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            self.id = generate_id(f"{self.owner}/{self.name}")

    def __repr__(self):
        return f"<Repository {self.owner}/{self.name}>"


class RepositoryAssociationModel(Base, AssociationLoaderMixin):
    __tablename__ = "repository_association"
    __table_args__ = (
        ForeignKeyConstraint(('repository_id',), ['repository.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
    )

    repository_id = Column(String, ForeignKey('repository.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)



import pandas as pd
from pathlib import Path

from sqlalchemy.orm import Session
from sqlalchemy.engine import Engine

from arepo.models.bf import BFClassModel, OperationModel, PhaseModel
from arepo.models.common.tag import TagModel
from arepo.models.common.platform.vendor import VendorModel
from arepo.models.common.weakness import (AbstractionModel, GroupingModel, CWEModel, CWEOperationModel, CWEPhaseModel,
                                          CWEBFClassModel)

from arepo.utils.misc import generate_id


tables_path = Path(__file__).parent.parent / 'tables'

TABLE_NAMES = {
    TagModel.__tablename__: TagModel, AbstractionModel.__tablename__: AbstractionModel,
    BFClassModel.__tablename__: BFClassModel, OperationModel.__tablename__: OperationModel,
    PhaseModel.__tablename__: PhaseModel, CWEModel.__tablename__: CWEModel, VendorModel.__tablename__: VendorModel
}


def populate(engine: Engine):
    """Clear the existing data and create new tables."""
    print('Initializing the database.')

    with Session(engine) as session, session.begin():

        if not session.query(AbstractionModel).all():
            abstractions_df = pd.read_csv(f'{tables_path}/abstractions.csv')
            session.add_all([AbstractionModel(**row.to_dict()) for i, row in abstractions_df.iterrows()])
            print("Populated 'abstractions' table.")

        if not session.query(TagModel).all():
            tags_df = pd.read_csv(f'{tables_path}/tags.csv')
            session.add_all([TagModel(**row.to_dict()) for i, row in tags_df.iterrows()])
            print("Populated 'tags' table.")

        if not session.query(OperationModel).all():
            operations_df = pd.read_csv(f'{tables_path}/operations.csv')
            session.add_all([OperationModel(**row.to_dict()) for i, row in operations_df.iterrows()])
            print("Populated 'operations' table.")

        if not session.query(PhaseModel).all():
            phases_df = pd.read_csv(f'{tables_path}/phases.csv')
            session.add_all([PhaseModel(**row.to_dict()) for i, row in phases_df.iterrows()])
            print("Populated 'phases' table.")

        if not session.query(BFClassModel).all():
            classes_df = pd.read_csv(f'{tables_path}/bf_classes.csv')
            session.add_all([BFClassModel(**row.to_dict()) for i, row in classes_df.iterrows()])
            print("Populated 'bf_classes' table.")

        if not session.query(CWEModel).all():
            cwes_df = pd.read_csv(f'{tables_path}/cwes.csv')
            session.add_all([CWEModel(**row.to_dict()) for i, row in cwes_df.iterrows()])
            print("Populated 'cwes' table.")

        if not session.query(CWEOperationModel).all():
            cwe_operations_df = pd.read_csv(f'{tables_path}/cwe_operation.csv')
            session.add_all([CWEOperationModel(**row.to_dict()) for i, row in cwe_operations_df.iterrows()])
            print("Populated 'cwe_operations' table.")

        if not session.query(CWEPhaseModel).all():
            cwe_phases_df = pd.read_csv(f'{tables_path}/cwe_phase.csv')
            session.add_all([CWEPhaseModel(**row.to_dict()) for i, row in cwe_phases_df.iterrows()])
            print("Populated 'cwe_phases' table.")

        if not session.query(CWEBFClassModel).all():
            cwe_bf_classes_df = pd.read_csv(f'{tables_path}/cwe_class.csv')
            session.add_all([CWEBFClassModel(**row.to_dict()) for i, row in cwe_bf_classes_df.iterrows()])
            print("Populated 'cwe_bf_classes' table.")

        if not session.query(VendorModel).all():
            vendors_df = pd.read_csv(f'{tables_path}/vendor_product_type.csv')['vendor'].unique()
            session.add_all([VendorModel(id=generate_id(vendor), name=vendor) for vendor in vendors_df])
            print("Populated 'vendors' table.")

        if not session.query(GroupingModel).all():
            grouping_df = pd.read_csv(f'{tables_path}/groupings.csv')
            session.add_all([GroupingModel(**row.to_dict()) for i, row in grouping_df.iterrows()])
            print("Populated 'grouping' table.")

        session.commit()



from arepo.base import Base
from arepo.mixins import EntityLoaderMixin, AssociationLoaderMixin

from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, ForeignKeyConstraint


class TagModel(Base, EntityLoaderMixin):
    __tablename__ = "tag"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)

    associations = relationship(
        "TagAssociationModel",
        back_populates='tag'
    )


class TagAssociationModel(Base, AssociationLoaderMixin):
    __tablename__ = 'tag_association'
    __table_args__ = (
        ForeignKeyConstraint(('tag_id',), ['tag.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
        ForeignKeyConstraint(('reference_id',), ['reference.id']),
    )

    tag_id = Column(Integer, ForeignKey('tag.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)
    reference_id = Column(String, ForeignKey('reference.id'), primary_key=True)

    tag = relationship("TagModel", back_populates="associations")
    source = relationship("SourceModel", back_populates="tags")
    reference = relationship("ReferenceModel", back_populates="tags")




from arepo.base import Base
from arepo.mixins import EntityLoaderMixin, AssociationLoaderMixin

from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, DateTime, ForeignKeyConstraint


class VulnerabilityModel(Base, EntityLoaderMixin):
    __tablename__ = "vulnerability"

    id = Column('id', String, primary_key=True)
    description = Column('description', String, nullable=True)
    published_date = Column('published_date', DateTime, nullable=False)
    last_modified_date = Column('last_modified_date', DateTime, nullable=False)
    configurations = relationship("ConfigurationModel", backref="vulnerability")
    source_id = Column(String, ForeignKey('source.id'), nullable=False)
    source = relationship("SourceModel", backref="vulnerabilities")

    # Relationship to ReferenceAssociationModel
    references = relationship(
        "ReferenceAssociationModel",
        back_populates="vulnerability"
    )

    cvss2 = relationship(
        "CVSS2AssociationModel",
        back_populates="vulnerability"
    )

    cvss3 = relationship(
        "CVSS3AssociationModel",
        back_populates="vulnerability"
    )


class VulnerabilityCWEModel(Base, AssociationLoaderMixin):
    __tablename__ = 'vulnerability_cwe'
    __table_args__ = (
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id']),
        ForeignKeyConstraint(('cwe_id',), ['cwe.id'])
    )

    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)
    cwe_id = Column(Integer, ForeignKey('cwe.id'), primary_key=True)



import pytest
from arepo.db import get_in_memory_database, DatabaseConnection


@pytest.fixture(scope="session")
def database_engine():
    return get_in_memory_database()


@pytest.fixture(scope="session")
def database_session(database_engine):
    db = DatabaseConnection("", database_engine)
    with db.get_session() as session:
        yield session



from arepo.base import Base
from arepo.mixins import EntityLoaderMixin, AssociationLoaderMixin
from arepo.utils.misc import generate_id

from sqlalchemy import (Column, Integer, String, Boolean, DateTime, ForeignKey, PrimaryKeyConstraint,
                        ForeignKeyConstraint)
from sqlalchemy.orm import relationship


class CommitModel(Base, EntityLoaderMixin):
    __tablename__ = "commit"

    id = Column('id', String, primary_key=True)
    sha = Column('sha', String, nullable=False)
    kind = Column('kind', String, nullable=False)
    date = Column('date', DateTime, nullable=True)
    state = Column('state', String, nullable=True)
    author = Column('author', String, nullable=True)
    message = Column('message', String, nullable=True)
    changes = Column('changes', Integer, nullable=True)
    available = Column('available', Boolean, nullable=True)
    additions = Column('additions', Integer, nullable=True)
    deletions = Column('deletions', Integer, nullable=True)
    files_count = Column('files_count', Integer, nullable=True)
    parents_count = Column('parents_count', Integer, nullable=True)
    repository_id = Column(String, ForeignKey('repository.id'))
    files = relationship("CommitFileModel", backref="commit")
    parents = relationship(
        "CommitModel",
        secondary="commit_parent",
        primaryjoin="CommitModel.id==CommitParentModel.commit_id",
        secondaryjoin="CommitModel.id==CommitParentModel.parent_id",
        backref="children"
    )

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the URL.
        """
        super().__init__(**kwargs)
        assert self.sha is not None, "sha must be provided."
        assert self.repository_id is not None, "repository_id must be provided."

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            self.id = generate_id(f"{self.repository_id}_{self.sha}")


class CommitAssociationModel(Base, AssociationLoaderMixin):
    __tablename__ = "commit_association"
    __table_args__ = (
        ForeignKeyConstraint(('commit_id',), ['commit.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
    )

    commit_id = Column(String, ForeignKey('commit.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)


class CommitFileModel(Base, EntityLoaderMixin):
    __tablename__ = "commit_file"

    id = Column('id', String, primary_key=True)
    filename = Column('filename', String, nullable=False)
    additions = Column('additions', Integer, nullable=False)
    deletions = Column('deletions', Integer, nullable=False)
    changes = Column('changes', Integer, nullable=False)
    status = Column('status', String, nullable=False)
    extension = Column('extension', String, nullable=True)
    patch = Column('patch', String, nullable=True)
    raw_url = Column('raw_url', String, nullable=True)
    commit_id = Column(String, ForeignKey('commit.id'))
    diff_blocks = relationship("DiffBlockModel", backref="commit_file")
    functions = relationship("FunctionModel", backref="commit_file")

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the URL.
        """
        super().__init__(**kwargs)
        assert self.filename is not None, "filename must be provided."
        assert self.commit_id is not None, "commit_id must be provided."

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            # TODO: probably it should be the sha from the commit instead of the commit_id
            self.id = generate_id(f"{self.commit_id}_{self.filename}")


class CommitParentModel(Base, EntityLoaderMixin):
    __tablename__ = "commit_parent"
    __table_args__ = (
        PrimaryKeyConstraint('commit_id', 'parent_id'),
    )

    commit_id = Column('commit_id', String, ForeignKey('commit.id'))
    parent_id = Column('parent_id', String, ForeignKey('commit.id'))



from os import environ
from arepo.db import DatabaseConnection
from arepo.models.vcs.core.repository import RepositoryAssociationModel
from arepo.models.vcs.core.commit import CommitAssociationModel

from sqlalchemy.orm import aliased


def get_overlap_vulnerability_ids(session, model, join_column, source_1, source_2):
    # Create aliases for the model
    alias_1 = aliased(model)
    alias_2 = aliased(model)

    # Perform the query
    return session.query(
        alias_1.vulnerability_id
    ).distinct().join(
        alias_2,
        (alias_1.vulnerability_id == alias_2.vulnerability_id)
    ).filter(
        alias_1.source_id == source_1,
        alias_2.source_id == source_2
    ).all()


db = DatabaseConnection(environ['SQLALCHEMY_DATABASE_URI'])
session = db.get_session(scoped=True)


print("CVEs with repositories in NVD and OSV",
      len(
          get_overlap_vulnerability_ids(
              session,
              model=RepositoryAssociationModel,
              join_column='repository_id',
              source_1='nvd_id',
              source_2='osv_id'
          )
      )
)

print("CVEs with commits in NVD and OSV",
      len(
          get_overlap_vulnerability_ids(
              session,
              model=CommitAssociationModel,
              join_column='commit_id',
              source_1='nvd_id',
              source_2='osv_id'
          )
      )
)

session.close()



from arepo.base import Base
from arepo.mixins import EntityLoaderMixin

from sqlalchemy import Column, String, ForeignKey, Boolean
from sqlalchemy.orm import relationship


class ConfigurationModel(Base, EntityLoaderMixin):
    __tablename__ = "configuration"

    id = Column('id', String, primary_key=True)
    vulnerability_id = Column('vulnerability_id', String, ForeignKey('vulnerability.id'), nullable=False)
    operator = Column('operator', String, nullable=True)
    is_vulnerable = Column('is_vulnerable', Boolean, nullable=True)
    is_multi_component = Column('is_multi_component', Boolean, nullable=True)
    is_platform_specific = Column('is_platform_specific', Boolean, nullable=True)

    nodes = relationship("NodeModel", back_populates="configuration")


class NodeModel(Base, EntityLoaderMixin):
    __tablename__ = "node"

    id = Column('id', String, primary_key=True)
    configuration_id = Column(String, ForeignKey('configuration.id'), nullable=False)
    operator = Column('operator', String, nullable=False)
    negate = Column('negate', Boolean, nullable=False)
    is_vulnerable = Column('is_vulnerable', Boolean, nullable=True)
    is_multi_component = Column('is_multi_component', Boolean, nullable=True)
    is_context_dependent = Column('is_context_dependent', Boolean, nullable=True)

    configuration = relationship("ConfigurationModel", back_populates="nodes")
    cpe_matches = relationship("CPEMatchModel", back_populates="node")



from arepo.models.vcs.core.commit import CommitModel
from arepo.models.source import SourceModel
from arepo.models.common.scoring import *
from arepo.models.common.platform import *
from arepo.models.common.vulnerability import *
from arepo.models.common.reference import *



import pytest

from sqlalchemy.exc import IntegrityError

from arepo.models.source import SourceModel
from arepo.models.common.scoring.cvss3 import CVSS3Model, CVSS3AssociationModel

# "https://services.nvd.nist.gov/rest/json/cves/2.0?cveId=CVE-2022-25854"
cvss3_data = [
    {
        "source": "nvd@nist.gov",
        "type": "Primary",
        "cvssData": {
            "version": "3.1",
            "vectorString": "CVSS:3.1/AV:N/AC:L/PR:L/UI:R/S:C/C:L/I:L/A:N",
            "attackVector": "NETWORK",
            "attackComplexity": "LOW",
            "privilegesRequired": "LOW",
            "userInteraction": "REQUIRED",
            "scope": "CHANGED",
            "confidentialityImpact": "LOW",
            "integrityImpact": "LOW",
            "availabilityImpact": "NONE",
            "baseScore": 5.4,
            "baseSeverity": "MEDIUM"
        },
        "exploitabilityScore": 2.3,
        "impactScore": 2.7
    },
    {
        "source": "report@snyk.io",
        "type": "Secondary",
        "cvssData": {
            "version": "3.1",
            "vectorString": "CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:L/I:L/A:N",
            "attackVector": "NETWORK",
            "attackComplexity": "LOW",
            "privilegesRequired": "NONE",
            "userInteraction": "REQUIRED",
            "scope": "UNCHANGED",
            "confidentialityImpact": "LOW",
            "integrityImpact": "LOW",
            "availabilityImpact": "NONE",
            "baseScore": 5.4,
            "baseSeverity": "MEDIUM"
        },
        "exploitabilityScore": 2.8,
        "impactScore": 2.5
    }
]


@pytest.mark.dependency(depends=["test_insert_source", "test_insert_vulnerability"])
@pytest.mark.dependency(name="test_insert_cvss3")
def test_insert_cvss3(database_session):
    for i, cvss_data in enumerate(cvss3_data):
        source_id = database_session.query(SourceModel).filter_by(email=cvss_data['source']).first().id

        try:
            cvss3_instance = CVSS3Model(
                id=f"test-cvss3-id-{i}",
                version=cvss_data['cvssData']['version'],
                vector_string=cvss_data['cvssData']['vectorString'],
                attack_vector=cvss_data['cvssData']['attackVector'],
                attack_complexity=cvss_data['cvssData']['attackComplexity'],
                privileges_required=cvss_data['cvssData']['privilegesRequired'],
                user_interaction=cvss_data['cvssData']['userInteraction'],
                scope=cvss_data['cvssData']['scope'],
                confidentiality_impact=cvss_data['cvssData']['confidentialityImpact'],
                integrity_impact=cvss_data['cvssData']['integrityImpact'],
                availability_impact=cvss_data['cvssData']['availabilityImpact'],
                base_severity=cvss_data['cvssData']['baseSeverity'],
                base_score=cvss_data['cvssData']['baseScore'],
                exploitability_score=cvss_data['exploitabilityScore'],
                impact_score=cvss_data['impactScore'],
            )
            database_session.add(cvss3_instance)
            database_session.commit()

            cvss3_association_instance = CVSS3AssociationModel(
                cvss_id=cvss3_instance.id,
                source_id=source_id,
                vulnerability_id="CVE-2022-25854"
            )

            database_session.add(cvss3_association_instance)
            database_session.commit()
        except IntegrityError:
            database_session.rollback()
            pytest.fail("IntegrityError occurred while inserting vulnerability")


@pytest.mark.dependency(depends=["test_insert_cvss3"])
def test_get_insert_cvss2(database_session):
    for i, cvss_data_v3 in enumerate(cvss3_data):
        cvss3_instance = database_session.query(CVSS3Model).filter_by(id=f"test-cvss3-id-{i}").first()
        assert cvss3_instance is not None
        source_id = database_session.query(SourceModel).filter_by(email=cvss_data_v3['source']).first().id

        cvss3_association_instance = (database_session.query(CVSS3AssociationModel)
                                      .filter_by(cvss_id=cvss3_instance.id)
                                      .filter_by(source_id=source_id)
                                      .filter_by(vulnerability_id="CVE-2022-25854")
                                      .first())

        assert cvss3_association_instance is not None



from sqlalchemy.inspection import inspect


class EntityLoaderMixin:
    """Mixin to handle session-based loading operations for regular models."""

    @classmethod
    def get_all_ids(cls, session):
        return set(id_tuple[0] for id_tuple in session.query(cls.id).all())

    @classmethod
    def load_all(cls, session):
        return session.query(cls).all()


class AssociationLoaderMixin:
    """Mixin to handle session-based loading operations for association models."""

    @property
    def composite_id(self):
        """Generate a composite ID based on the primary key fields."""
        mapper = inspect(self.__class__)  # Get the mapper for the class, not the instance
        primary_key_values = [getattr(self, key.name) for key in mapper.primary_key]
        return "_".join(str(value) for value in primary_key_values)

    @classmethod
    def get_all_ids(cls, session):
        """Generate composite keys for association models using primary key fields."""
        primary_keys = [key.name for key in inspect(cls).primary_key]

        # Query all the rows in the association table
        rows = session.query(cls).all()

        # Create a set of composite keys based on the primary key fields
        composite_keys = set(
            "_".join(str(getattr(row, key)) for key in primary_keys)
            for row in rows
        )

        return composite_keys



from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String
from arepo.base import Base


class OperationModel(Base):
    __tablename__ = "operation"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    cwes = relationship('CWEModel', secondary="cwe_operation", backref='operations')


class PhaseModel(Base):
    __tablename__ = "phase"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    acronym = Column('acronym', String, nullable=False)
    url = Column('url', String, nullable=True)
    cwes = relationship('CWEModel', secondary="cwe_phase", backref='phases')


class BFClassModel(Base):
    __tablename__ = "bf_class"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    url = Column('url', String, nullable=True)
    cwes = relationship('CWEModel', secondary="cwe_bf_class", backref='bf_classes')



from arepo.utils.misc import generate_id

from arepo.base import Base
from arepo.mixins import EntityLoaderMixin, AssociationLoaderMixin

from sqlalchemy.orm import relationship
from sqlalchemy import Column, String, ForeignKey, ForeignKeyConstraint


class ReferenceModel(Base, EntityLoaderMixin):
    __tablename__ = "reference"

    id = Column('id', String, primary_key=True)
    url = Column('url', String, nullable=False)

    # Relationship to ReferenceAssociationModel
    associations = relationship(
        'ReferenceAssociationModel',
        back_populates='reference'
    )

    tags = relationship(
        'TagAssociationModel',
        back_populates='reference'
    )

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the URL.
        """
        super().__init__(**kwargs)
        assert self.url is not None, "URL must be provided."

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            self.id = generate_id(self.url)


class ReferenceAssociationModel(Base, AssociationLoaderMixin):
    __tablename__ = "reference_association"
    __table_args__ = (
        ForeignKeyConstraint(('reference_id',), ['reference.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id']),
    )

    reference_id = Column(String, ForeignKey('reference.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)

    # Define the relationships
    vulnerability = relationship("VulnerabilityModel", back_populates="references")
    reference = relationship("ReferenceModel", back_populates="associations")
    source = relationship("SourceModel", back_populates="references")



from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, Enum

from arepo.base import Base
from arepo.utils.misc import generate_id
from arepo.mixins import EntityLoaderMixin


change_type = Enum('addition', 'deletion', name='change_type')


class ChangeModel(Base, EntityLoaderMixin):
    __tablename__ = 'change'
    id = Column(String, primary_key=True)
    line = Column('number', Integer, nullable=False)
    content = Column('content', String, nullable=False)
    start_col = Column(Integer)
    end_col = Column(Integer)
    type = Column('type', change_type)
    diff_block_id = Column(String, ForeignKey('diff_block.id'))

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from attributes.
        """
        super().__init__(**kwargs)
        assert self.diff_block_id is not None, "diff_block_id must be provided."
        assert self.type is not None, "type must be provided."
        assert self.line is not None, "line must be provided."
        assert self.start_col is not None, "start_col must be provided."

        if self.id is None:
            self.id = generate_id(f"{self.diff_block_id}_{self.type}_{self.line}_{self.start_col}")


class DiffBlockModel(Base, EntityLoaderMixin):
    __tablename__ = 'diff_block'
    id = Column(String, primary_key=True)
    order = Column(Integer)
    changes = relationship("ChangeModel", backref="diff_block")
    a_path = Column('a_path', String, nullable=False)
    commit_file_id = Column(String, ForeignKey('commit_file.id'))

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from attributes.
        """
        super().__init__(**kwargs)
        assert self.commit_file_id is not None, "commit_file_id must be provided."
        assert self.a_path is not None, "a_path must be provided."
        assert self.order is not None, "order must be provided."

        if self.id is None:
            self.id = generate_id(f"{self.commit_file_id}_{self.a_path}_{self.order}")



from arepo.base import Base
from arepo.mixins import EntityLoaderMixin

from sqlalchemy import Column, Integer, String, ForeignKey, ForeignKeyConstraint, PrimaryKeyConstraint
from sqlalchemy.orm import relationship


class AbstractionModel(Base):
    __tablename__ = "abstraction"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    cwes = relationship('CWEModel', backref='abstraction')


class GroupingModel(Base):
    __tablename__ = "grouping"
    __table_args__ = (
        PrimaryKeyConstraint('parent_id', 'child_id'),
    )

    parent_id = Column('parent_id', Integer, ForeignKey('cwe.id'))
    child_id = Column('child_id', Integer, ForeignKey('cwe.id'))


class CWEModel(Base, EntityLoaderMixin):
    __tablename__ = "cwe"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    url = Column('url', String, nullable=False)
    abstraction_id = Column(Integer, ForeignKey('abstraction.id'))
    vulnerabilities = relationship('VulnerabilityModel', secondary="vulnerability_cwe", backref='cwes')

    def to_dict(self):
        return {'id': self.id, 'name': self.name, 'url': self.url, 'abstraction_id': self.abstraction_id}


class CWEOperationModel(Base):
    __tablename__ = "cwe_operation"
    __table_args__ = (
        ForeignKeyConstraint(('cwe_id',), ['cwe.id']),
        ForeignKeyConstraint(('operation_id',), ['operation.id'])
    )

    cwe_id = Column(Integer, ForeignKey('cwe.id'), primary_key=True)
    operation_id = Column(Integer, ForeignKey('operation.id'), primary_key=True)


class CWEPhaseModel(Base):
    __tablename__ = "cwe_phase"
    __table_args__ = (
        ForeignKeyConstraint(('cwe_id',), ['cwe.id']),
        ForeignKeyConstraint(('phase_id',), ['phase.id']),
    )

    cwe_id = Column(Integer, ForeignKey('cwe.id'), primary_key=True)
    phase_id = Column(Integer, ForeignKey('phase.id'), primary_key=True)


class CWEBFClassModel(Base):
    __tablename__ = "cwe_bf_class"
    __table_args__ = (
        ForeignKeyConstraint(('cwe_id',), ['cwe.id']),
        ForeignKeyConstraint(('bf_class_id',), ['bf_class.id']),
    )

    cwe_id = Column(Integer, ForeignKey('cwe.id'), primary_key=True)
    bf_class_id = Column(Integer, ForeignKey('bf_class.id'), primary_key=True)



from arepo.base import Base
from arepo.mixins import EntityLoaderMixin
from arepo.utils.misc import generate_id

from sqlalchemy.orm import relationship
from sqlalchemy import Column, String, ForeignKey


class ProductModel(Base, EntityLoaderMixin):
    __tablename__ = "product"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    part = Column('part', String, nullable=False)
    vendor_id = Column(String, ForeignKey('vendor.id'), nullable=False)
    # TODO: add more fields
    vendor = relationship("VendorModel", back_populates="products")
    cpes = relationship("CPEModel", back_populates="product")

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the columns.
        """
        super().__init__(**kwargs)

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            # Get only the defined attributes from the class using __table__.columns
            attributes = {
                col.name: str(getattr(self, col.name)) for col in self.__table__.columns
                if getattr(self, col.name) is not None
            }

            # Create a string representation of the attributes sorted by key
            sorted_attributes_str = ''.join(f"{key}={value}" for key, value in sorted(attributes.items()))

            self.id = generate_id(sorted_attributes_str)



import pytest
import dateutil.parser as date_parser

from sqlalchemy.exc import IntegrityError

from arepo.models.source import SourceModel
from arepo.models.vcs.core.commit import CommitModel, CommitFileModel, CommitAssociationModel


commit_data = {
    'id': 'test-commit-id',
    'sha': '198c0451fad188390390395ccfc84ab371def4c7',
    'url': 'https://api.github.com/repos/yairEO/tagify/commits/198c0451fad188390390395ccfc84ab371def4c7',
    'kind': 'patch',
    'date': date_parser.parse('2022-02-17T08:16:09Z'),
    'state': 'pending',
    'author': 'author',
    'message': 'fixes #989 - fix XSS',
    'changes': 2,
    'available': True,
    'additions': 1,
    'deletions': 1,
    'files_count': 1,
    'parents_count': 1,
    'repository_id': 'test-id',
    'vulnerability_id': 'CVE-2022-25854'
}


commit_file_data = {
    'id': 'test-commit-file-id',
    "filename": "src/tagify.js",
    "additions": 1,
    "deletions": 1,
    "changes": 2,
    "status": "modified",
    'extension': 'js',
    "patch": "@@ -101,7 +101,7 @@ Tagify.prototype = {\n \n         _s.disabled = input.hasAttribute('disabled')\n         _s.readonly = _s.readonly || input.hasAttribute('readonly')\n-        _s.placeholder = input.getAttribute('placeholder') || _s.placeholder || \"\"\n+        _s.placeholder = escapeHTML(input.getAttribute('placeholder') || _s.placeholder || \"\")\n         _s.required = input.hasAttribute('required')\n \n         for( let name in _s.classNames )",
    "raw_url": "https://github.com/yairEO/tagify/raw/198c0451fad188390390395ccfc84ab371def4c7/src%2Ftagify.js",
    'commit_id': commit_data['id']
}


@pytest.mark.dependency(depends=["insert_repository"])
@pytest.mark.dependency(depends=["test_insert_vulnerability"])
@pytest.mark.dependency(name="test_insert_commit")
def test_insert_commit(database_session):
    try:
        # Creating VulnerabilityModel object
        commit = CommitModel(
            id=commit_data['id'],
            sha=commit_data['sha'],
            kind=commit_data['kind'],
            date=commit_data['date'],
            state=commit_data['state'],
            author=commit_data['author'],
            message=commit_data['message'],
            changes=commit_data['changes'],
            available=commit_data['available'],
            additions=commit_data['additions'],
            deletions=commit_data['deletions'],
            files_count=commit_data['files_count'],
            parents_count=commit_data['parents_count'],
            repository_id=commit_data['repository_id']
        )

        # Inserting the object into the database
        database_session.add(commit)
        database_session.commit()

        source_id = database_session.query(SourceModel).filter_by(email="report@snyk.io").first().id
        # Creating CommitAssociationModel object

        commit_association = CommitAssociationModel(
            commit_id=commit_data['id'],
            vulnerability_id=commit_data['vulnerability_id'],
            source_id=source_id
        )

        database_session.add(commit_association)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting vulnerability")


@pytest.mark.dependency(depends=["test_insert_commit"])
def test_insert_commit_file(database_session):
    try:
        # Creating VulnerabilityModel object
        commit_file = CommitFileModel(
            id=commit_file_data['id'],
            filename=commit_file_data['filename'],
            additions=commit_file_data['additions'],
            deletions=commit_file_data['deletions'],
            changes=commit_file_data['changes'],
            status=commit_file_data['status'],
            extension=commit_file_data['extension'],
            patch=commit_file_data['patch'],
            raw_url=commit_file_data['raw_url'],
            commit_id=commit_data['id']
        )

        # Inserting the object into the database
        database_session.add(commit_file)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting vulnerability")


@pytest.mark.dependency(depends=["test_insert_commit"])
def test_query_commit(database_session):
    result = database_session.query(CommitModel).filter(CommitModel.id == commit_data['id']).first()

    assert result.id == commit_data['id']
    assert result.sha == commit_data['sha']
    assert result.kind == commit_data['kind']
    assert result.date.date() == commit_data['date'].date()
    assert result.state == commit_data['state']
    assert result.author == commit_data['author']
    assert result.message == commit_data['message']
    assert result.changes == commit_data['changes']
    assert result.available == commit_data['available']
    assert result.additions == commit_data['additions']
    assert result.deletions == commit_data['deletions']
    assert result.files_count == commit_data['files_count']
    assert result.parents_count == commit_data['parents_count']
    assert result.repository_id == commit_data['repository_id']

    source_id = database_session.query(SourceModel).filter_by(email="report@snyk.io").first().id
    association = (database_session.query(CommitAssociationModel)
                   .filter_by(commit_id=commit_data['id'])
                   .filter_by(vulnerability_id=commit_data['vulnerability_id'])
                   .filter_by(source_id=source_id)
                   .first())

    assert association.commit_id is not None


@pytest.mark.dependency(depends=["test_insert_commit_file"])
def test_query_commit_file(database_session):
    result = database_session.query(CommitFileModel).filter(CommitFileModel.id == commit_file_data['id']).first()

    assert result.id == commit_file_data['id']
    assert result.filename == commit_file_data['filename']
    assert result.additions == commit_file_data['additions']
    assert result.deletions == commit_file_data['deletions']
    assert result.changes == commit_file_data['changes']
    assert result.status == commit_file_data['status']
    assert result.extension == commit_file_data['extension']
    assert result.patch == commit_file_data['patch']
    assert result.raw_url == commit_file_data['raw_url']
    assert result.commit_id == commit_data['id']



from arepo.base import Base

from sqlalchemy import Column, String
from sqlalchemy.orm import relationship


class SourceModel(Base):
    __tablename__ = "source"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    email = Column('email', String, nullable=False)

    # Relationship to ReferenceAssociationModel
    references = relationship(
        'ReferenceAssociationModel',
        back_populates='source'
    )

    tags = relationship(
        'TagAssociationModel',
        back_populates='source'
    )

    cvss2 = relationship(
        'CVSS2AssociationModel',
        back_populates='source'
    )

    cvss3 = relationship(
        'CVSS3AssociationModel',
        back_populates='source'
    )



import unittest
from arepo.db import get_in_memory_database, DatabaseConnection
from arepo.models.common.tag import TagModel


class TestInMemoryDatabaseQuery(unittest.TestCase):
    def test_query_in_memory_database(self):
        # Initialize the in-memory database and get the engine
        engine = get_in_memory_database()

        # Create the database connection
        db = DatabaseConnection("", engine)

        # Create a session
        session = db.get_session()

        # Query the TagModel table
        entries = session.query(TagModel).all()

        # Close the session
        session.close()

        # Define the expected output
        expected_output = [
            (1, 'Third Party Advisory'),
            (2, 'Mailing List'),
            (3, 'Permissions Required'),
            (4, 'Tool Signature'),
            (5, 'VDB Entry'),
            (6, 'Release Notes'),
            (7, 'Patch'),
            (8, 'Technical Description'),
            (9, 'Press/Media Coverage'),
            (10, 'Exploit'),
            (11, 'US Government Resource'),
            (12, 'Broken Link'),
            (13, 'Not Applicable'),
            (14, 'Product'),
            (15, 'URL Repurposed'),
            (16, 'Vendor Advisory'),
            (17, 'Mitigation'),
            (18, 'Issue Tracking'),
            (19, 'Other')
        ]

        # Convert the entries to a list of tuples for comparison
        actual_output = [(entry.id, entry.name) for entry in entries]

        # Check if the actual output matches the expected output
        self.assertEqual(actual_output, expected_output)


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import patch
from io import StringIO
from arepo.main import main


class TestInitCommand(unittest.TestCase):
    @patch('sys.stdout', new_callable=StringIO)
    def test_init_command(self, mock_stdout):
        db_uri = "sqlite:///:memory:"
        args = ['--uri', db_uri, 'init']

        # Call the main function with arguments
        with patch('sys.argv', ['arepo'] + args):
            main()

        expected_output = (
            "Initializing the database.\n"
            "Populated 'abstractions' table.\n"
            "Populated 'tags' table.\n"
            "Populated 'operations' table.\n"
            "Populated 'phases' table.\n"
            "Populated 'bf_classes' table.\n"
            "Populated 'cwes' table.\n"
            "Populated 'cwe_operations' table.\n"
            "Populated 'cwe_phases' table.\n"
            "Populated 'cwe_bf_classes' table.\n"
            "Populated 'vendors' table.\n"
            "Populated 'grouping' table.\n"
        )

        self.assertEqual(mock_stdout.getvalue(), expected_output)


if __name__ == "__main__":
    unittest.main()



import pytest

from sqlalchemy.exc import IntegrityError

from arepo.models.source import SourceModel
from arepo.models.vcs.core.repository import RepositoryModel, RepositoryAssociationModel


# Repo data
repository_data = {
    'id': 'test-id',
    "name": 'tagify',
    "owner": 'yairEO',
    'available': True,
    'description': 'tags input component',
    'language': 'JavaScript',
    'size': 5410,
    'watchers': 20,
    'forks': 420,
    'stargazers': 3225
}


@pytest.mark.dependency(name="insert_repository")
def test_insert_repository(database_session):
    try:
        # Creating VulnerabilityModel object
        repository = RepositoryModel(
            id=repository_data['id'],
            name=repository_data['name'],
            owner=repository_data['owner'],
            available=repository_data['available'],
            description=repository_data['description'],
            language=repository_data['language'],
            size=repository_data['size'],
            watchers=repository_data['watchers'],
            forks=repository_data['forks'],
            stargazers=repository_data['stargazers']
        )

        # Inserting the object into the database
        database_session.add(repository)
        database_session.commit()

        source_id = database_session.query(SourceModel).filter_by(email="report@snyk.io").first().id

        # Creating RepositoryAssociationModel object
        repository_association = RepositoryAssociationModel(
            repository_id=repository_data['id'],
            vulnerability_id='CVE-2022-25854',
            source_id=source_id
        )

        # Inserting the object into the database
        database_session.add(repository_association)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting vulnerability")


@pytest.mark.dependency(depends=["insert_repository"])
def test_query_repository(database_session):
    # Querying the database for the object
    result = database_session.query(RepositoryModel).filter(RepositoryModel.id == repository_data['id']).first()

    # Asserting the result
    assert result.id == repository_data['id']
    assert result.name == repository_data['name']
    assert result.owner == repository_data['owner']
    assert result.available == repository_data['available']
    assert result.description == repository_data['description']
    assert result.language == repository_data['language']
    assert result.size == repository_data['size']
    assert result.watchers == repository_data['watchers']
    assert result.forks == repository_data['forks']
    assert result.stargazers == repository_data['stargazers']

    # Querying the database for the association object
    source_id = database_session.query(SourceModel).filter_by(email="report@snyk.io").first().id
    result = (database_session.query(RepositoryAssociationModel)
              .filter_by(repository_id=repository_data['id'])
              .filter_by(source_id=source_id)
              .filter_by(vulnerability_id='CVE-2022-25854')
              .first())

    # Asserting the result
    assert result is not None




from arepo.base import Base
from arepo.mixins import EntityLoaderMixin, AssociationLoaderMixin
from arepo.utils.misc import generate_id

from sqlalchemy.orm import relationship
from sqlalchemy import Column, String, ForeignKey, Float, ForeignKeyConstraint


class CVSS3Model(Base, EntityLoaderMixin):
    __tablename__ = "cvss3"

    id = Column(String, primary_key=True)
    version = Column('version', String)
    vector_string = Column('vector_string', String)
    attack_vector = Column('attack_vector', String)
    attack_complexity = Column('attack_complexity', String)
    privileges_required = Column('privileges_required', String)
    user_interaction = Column('user_interaction', String)
    scope = Column('scope', String)
    confidentiality_impact = Column('confidentiality_impact', String)
    integrity_impact = Column('integrity_impact', String)
    availability_impact = Column('availability_impact', String)
    base_severity = Column('base_severity', String, nullable=True)
    base_score = Column('base_score', Float, nullable=True)
    exploitability_score = Column('exploitability_score', Float, nullable=True)
    impact_score = Column('impact_score', Float, nullable=True)
    # type = Column('type', String, nullable=True)  # TODO: to be added later

    associations = relationship(
        "CVSS3AssociationModel",
        back_populates="cvss"
    )

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the columns.
        """
        super().__init__(**kwargs)

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            # Get only the defined attributes from the class using __table__.columns
            attributes = {
                col.name: str(getattr(self, col.name)) for col in self.__table__.columns
                if getattr(self, col.name) is not None
            }

            # Create a string representation of the attributes sorted by key
            sorted_attributes_str = ''.join(f"{key}={value}" for key, value in sorted(attributes.items()))

            self.id = generate_id(sorted_attributes_str)


# TODO: there must be a better way to handle this
class CVSS3AssociationModel(Base, AssociationLoaderMixin):
    __tablename__ = 'cvss3_association'
    __table_args__ = (
        ForeignKeyConstraint(('cvss_id',), ['cvss3.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id'])
    )

    cvss_id = Column(String, ForeignKey('cvss3.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)

    cvss = relationship("CVSS3Model", back_populates="associations")
    source = relationship("SourceModel", back_populates="cvss3")
    vulnerability = relationship("VulnerabilityModel", back_populates="cvss3")




import argparse
from arepo.db import DatabaseConnection
from arepo.utils.db import TABLE_NAMES


def main():
    parser = argparse.ArgumentParser("Dataset interaction commands")
    parser.add_argument('-u', '--uri', help='Database URI', required=True)
    subparsers = parser.add_subparsers(dest='subparser')
    init_parser = subparsers.add_parser('init', help='Initialize the database')
    list_parser = subparsers.add_parser('list', help='List content in tables')
    list_parser.add_argument('-t', '--table', choices=list(TABLE_NAMES.keys()), help='Table to list')
    list_parser.add_argument('-l', '--limit', type=int, help='Limit the number of entries', default=20)

    args = parser.parse_args()

    if args.subparser == 'init':
        DatabaseConnection.init(args.uri)

    if args.subparser == 'list':
        if args.limit < 1:
            print('Limit must be greater than 0.')
            return

        print('Getting database connection.')
        db = DatabaseConnection(args.uri)
        print('Getting session.')
        session = db.get_session(scoped=True)

        print(f'Querying {args.table}.')
        entries = session.query(TABLE_NAMES[args.table]).all()
        # keep the first 20 entries only
        if len(entries) > args.limit:
            entries = entries[:args.limit]
            print(f'Only the first {args.limit} entries are shown.')

        for entry in entries:
            print(entry.id, entry.name)

        session.close()



from arepo.base import Base
from arepo.mixins import EntityLoaderMixin
from arepo.utils.misc import generate_id

from sqlalchemy import Column, String, ForeignKey, Boolean
from sqlalchemy.orm import relationship


class CPEModel(Base, EntityLoaderMixin):
    __tablename__ = "cpe"

    id = Column('id', String, primary_key=True)
    product_id = Column(String, ForeignKey('product.id'), nullable=False)
    version = Column(String, nullable=True, default="*")
    update = Column(String, nullable=True, default="*")
    edition = Column(String, nullable=True, default="*")
    language = Column(String, nullable=True, default="*")
    sw_edition = Column(String, nullable=True, default="*")
    target_sw = Column(String, nullable=True, default="*")
    target_hw = Column(String, nullable=True, default="*")
    other = Column(String, nullable=True, default="*")

    product = relationship("ProductModel", back_populates="cpes")
    cpe_matches = relationship("CPEMatchModel", back_populates="cpe")

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the columns.
        """
        super().__init__(**kwargs)

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            # Get only the defined attributes from the class using __table__.columns
            attributes = {}

            for col in self.__table__.columns:
                if col.name == 'id':
                    continue

                if getattr(self, col.name) is None:
                    # Set the default value, to ensure consistency
                    attributes[col.name] = col.default.arg
                else:
                    attributes[col.name] = str(getattr(self, col.name))

            # Create a string representation of the attributes sorted by key
            sorted_attributes_str = ''.join(f"{key}={value}" for key, value in sorted(attributes.items()))

            self.id = generate_id(sorted_attributes_str)


class CPEMatchModel(Base, EntityLoaderMixin):
    __tablename__ = "cpe_match"

    id = Column('id', String, primary_key=True)
    cpe_id = Column(String, ForeignKey('cpe.id'), nullable=False)
    node_id = Column(String, ForeignKey('node.id'), nullable=False)
    vulnerable = Column(Boolean, nullable=False)
    version_start_including = Column(String, nullable=True)
    version_start_excluding = Column(String, nullable=True)
    version_end_including = Column(String, nullable=True)
    version_end_excluding = Column(String, nullable=True)

    cpe = relationship("CPEModel", back_populates="cpe_matches")
    node = relationship("NodeModel", back_populates="cpe_matches")



import pytest

from sqlalchemy.exc import IntegrityError

from arepo.models.source import SourceModel
from arepo.models.common.scoring.cvss2 import CVSS2Model, CVSS2AssociationModel


cvss2_data = [
    {
        "source": "nvd@nist.gov",
        "type": "Primary",
        "cvssData": {
            "version": "2.0",
            "vectorString": "AV:N/AC:M/Au:S/C:N/I:P/A:N",
            "accessVector": "NETWORK",
            "accessComplexity": "MEDIUM",
            "authentication": "SINGLE",
            "confidentialityImpact": "NONE",
            "integrityImpact": "PARTIAL",
            "availabilityImpact": "NONE",
            "baseScore": 3.5
        },
        "baseSeverity": "LOW",
        "exploitabilityScore": 6.8,
        "impactScore": 2.9,
        "acInsufInfo": False,
        "obtainAllPrivilege": False,
        "obtainUserPrivilege": False,
        "obtainOtherPrivilege": False,
        "userInteractionRequired": True
    }
]


@pytest.mark.dependency(depends=["test_insert_source", "test_insert_vulnerability"])
@pytest.mark.dependency(name="test_insert_cvss2")
def test_insert_cvss2(database_session):
    for i, cvss_data_v2 in enumerate(cvss2_data):
        source_id = database_session.query(SourceModel).filter_by(email=cvss_data_v2['source']).first().id

        try:
            cvss2_instance = CVSS2Model(
                id=f"test-cvss2-id-{i}",
                vector_string=cvss_data_v2['cvssData']['vectorString'],
                access_vector=cvss_data_v2['cvssData']['accessVector'],
                access_complexity=cvss_data_v2['cvssData']['accessComplexity'],
                authentication=cvss_data_v2['cvssData']['authentication'],
                confidentiality_impact=cvss_data_v2['cvssData']['confidentialityImpact'],
                integrity_impact=cvss_data_v2['cvssData']['integrityImpact'],
                availability_impact=cvss_data_v2['cvssData']['availabilityImpact'],
                base_severity=cvss_data_v2['baseSeverity'],
                base_score=cvss_data_v2['cvssData']['baseScore'],
                exploitability_score=cvss_data_v2['exploitabilityScore'],
                impact_score=cvss_data_v2['impactScore'],
                ac_insuf_info=cvss_data_v2['acInsufInfo'],
                obtain_all_privilege=cvss_data_v2['obtainAllPrivilege'],
                obtain_user_privilege=cvss_data_v2['obtainUserPrivilege'],
                obtain_other_privilege=cvss_data_v2['obtainOtherPrivilege'],
                user_interaction_required=cvss_data_v2['userInteractionRequired']
            )
            database_session.add(cvss2_instance)
            database_session.commit()

            cvss2_association_instance = CVSS2AssociationModel(
                cvss_id=cvss2_instance.id,
                source_id=source_id,
                vulnerability_id="CVE-2022-25854"
            )

            database_session.add(cvss2_association_instance)
            database_session.commit()
        except IntegrityError:
            database_session.rollback()
            pytest.fail("IntegrityError occurred while inserting vulnerability")


@pytest.mark.dependency(depends=["test_insert_cvss2"])
def test_get_insert_cvss2(database_session):
    for i, cvss_data_v2 in enumerate(cvss2_data):
        cvss2_instance = database_session.query(CVSS2Model).filter_by(id=f"test-cvss2-id-{i}").first()
        assert cvss2_instance is not None
        source_id = database_session.query(SourceModel).filter_by(email=cvss_data_v2['source']).first().id

        cvss2_association_instance = (database_session.query(CVSS2AssociationModel)
                                      .filter_by(cvss_id=cvss2_instance.id)
                                      .filter_by(source_id=source_id)
                                      .filter_by(vulnerability_id="CVE-2022-25854")
                                      .first())

        assert cvss2_association_instance is not None



import pytest
from datetime import datetime
from arepo.models.common.vulnerability import VulnerabilityModel
from arepo.models.source import SourceModel


# Mocked vulnerability data
vulnerability_data = {
    "cve": {
        "id": "CVE-2022-25854",
        "sourceIdentifier": "report@snyk.io",
        "published": "2022-04-29T20:15:07.580",
        "lastModified": "2022-09-23T18:57:11.480",
        "vulnStatus": "Analyzed",
        "descriptions": [
            {
                "lang": "en",
                "value": "This affects the package @yaireo/tagify before 4.9.8. The package is used for rendering UI "
                         "components inside the input or text fields, and an attacker can pass a malicious placeholder "
                         "value to it to fire the XSS payload."
            }
        ]
    }
}


@pytest.mark.dependency(depends=["test_insert_sources"])
@pytest.mark.dependency(name="test_insert_vulnerability")
def test_insert_vulnerability(database_session):
    from sqlalchemy.exc import IntegrityError
    email = vulnerability_data["cve"]["sourceIdentifier"]
    source_id = database_session.query(SourceModel).filter_by(email=email).first().id

    try:
        # Creating VulnerabilityModel object
        vulnerability = VulnerabilityModel(
            id=vulnerability_data["cve"]["id"],
            description=vulnerability_data["cve"]["descriptions"][0]["value"],
            source_id=source_id,
            published_date=datetime.fromisoformat(vulnerability_data["cve"]["published"]),
            last_modified_date=datetime.fromisoformat(vulnerability_data["cve"]["lastModified"])
        )

        database_session.add(vulnerability)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting vulnerability")


@pytest.mark.dependency(depends=["test_insert_vulnerability"])
def test_get_inserted_vulnerability(database_session):
    # Query the database to retrieve the inserted vulnerability
    retrieved_vulnerability = database_session.query(VulnerabilityModel).filter_by(id="CVE-2022-25854").first()

    # Check if the retrieved vulnerability exists
    assert retrieved_vulnerability is not None
    # Assert the retrieved data matches the expected values
    assert retrieved_vulnerability.id == vulnerability_data["cve"]["id"]
    assert retrieved_vulnerability.description == vulnerability_data["cve"]["descriptions"][0]["value"]
    assert retrieved_vulnerability.source.email == vulnerability_data["cve"]["sourceIdentifier"]


@pytest.mark.dependency(depends=["test_insert_vulnerability"])
def test_get_all_ids_mixin(database_session):
    # Retrieve all vulnerability IDs from the database
    vulnerability_ids = VulnerabilityModel.get_all_ids(database_session)

    # Check if the inserted vulnerability ID is in the retrieved IDs
    assert "CVE-2022-25854" in vulnerability_ids



from arepo.base import Base
from arepo.mixins import EntityLoaderMixin
from arepo.utils.misc import generate_id

from sqlalchemy.orm import relationship
from sqlalchemy import Column, String


class VendorModel(Base, EntityLoaderMixin):
    __tablename__ = "vendor"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    # TODO: add more fields
    products = relationship("ProductModel", back_populates="vendor")

    def __init__(self, **kwargs):
        """
            If the ID is not provided, it will be generated from the name.
        """
        super().__init__(**kwargs)
        assert self.name is not None, "URL must be provided."

        if self.id is None:
            # TODO: should be defined as read-only in the schema to avoid issues with future changes
            self.id = generate_id(self.name)



import pytest

from sqlalchemy.exc import IntegrityError

from arepo.models.common.platform.product import ProductModel
from arepo.models.common.platform.vendor import VendorModel
from arepo.models.common.platform.cpe import CPEMatchModel, CPEModel
from arepo.models.common.platform.configuration import ConfigurationModel, NodeModel


config_data = [
    {
        "id": "config_id",
        "is_vulnerable": True,
        "is_multi_component": False,
        "is_platform_specific": False
    }
]

node_data = [
    {
        "id": "node_id",
        "configuration_id": "config_id",
        "operator": "OR",
        "negate": False,
        "is_vulnerable": True,
        "is_multi_component": False,
        "is_context_dependent": False
    }
]

cpe_match_data = [
    {
        "cpe_id": "cpe_id",
        "node_id": "node_id",
        "vulnerable": True,
        "versionEndExcluding": "4.9.8",
        "matchCriteriaId": "135622D7-7891-420D-81D9-C078AD572C78"
    }
]

cpe_data = [
    {
        "id": "cpe_id",
        "criteria": ":a:tagify_project:tagify:*:*:*:*:*:*:*:*"
    }
]


@pytest.mark.dependency(name="test_platform_configuration")
def test_platform_configuration(database_session):
    for config in config_data:
        try:
            configuration = ConfigurationModel(
                id=config['id'],
                vulnerability_id="CVE-2022-25854",
                is_vulnerable=config['is_vulnerable'],
                is_multi_component=config['is_multi_component'],
                is_platform_specific=config['is_platform_specific']
            )

            database_session.add(configuration)
            database_session.commit()
        except IntegrityError:
            database_session.rollback()
            pytest.fail("IntegrityError occurred while inserting configuration")


@pytest.mark.dependency(name="test_platform_vendor")
def test_platform_vendor(database_session):
    try:
        vendor = VendorModel(
            id="vendor_id",
            name="tagify_project"
        )

        database_session.add(vendor)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting vendor")


@pytest.mark.dependency(depends=["test_platform_vendor"])
@pytest.mark.dependency(name="test_platform_product")
def test_platform_product(database_session):
    try:
        vendor_id = database_session.query(VendorModel).filter_by(name="tagify_project").first().id

        product = ProductModel(
            id="product_id",
            part="a",
            name="tagify",
            vendor_id=vendor_id
        )

        database_session.add(product)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting product")


@pytest.mark.dependency(depends=["test_platform_product"])
@pytest.mark.dependency(name="test_cpe")
def test_cpe(database_session):
    try:
        product_id = database_session.query(ProductModel).filter_by(name="tagify").first().id

        cpe = CPEModel(
            id="cpe_id",
            product_id=product_id
        )

        database_session.add(cpe)
        database_session.commit()
    except IntegrityError:
        database_session.rollback()
        pytest.fail("IntegrityError occurred while inserting CPE")


@pytest.mark.dependency(depends=["test_platform_configuration"])
@pytest.mark.dependency(name="test_configuration_node")
def test_configuration_node(database_session):
    for node in node_data:
        config_id = database_session.query(ConfigurationModel).filter_by(id="config_id").first().id

        try:
            node = NodeModel(
                id="node_id",
                configuration_id=config_id,
                operator=node['operator'],
                negate=node['negate'],
                is_vulnerable=node['is_vulnerable'],
                is_multi_component=node['is_multi_component'],
                is_context_dependent=node['is_context_dependent']
            )

            database_session.add(node)
            database_session.commit()
        except IntegrityError:
            database_session.rollback()
            pytest.fail("IntegrityError occurred while inserting node")


@pytest.mark.dependency(depends=["test_cpe", "test_configuration_node"])
@pytest.mark.dependency(name="test_cpe_match")
def test_cpe_match(database_session):
    for cpe_match in cpe_match_data:
        cpe_id = database_session.query(CPEModel).filter_by(id="cpe_id").first().id
        node_id = database_session.query(NodeModel).filter_by(id="node_id").first().id

        try:
            cpe_match = CPEMatchModel(
                id=cpe_match['matchCriteriaId'],
                cpe_id=cpe_id,
                node_id=node_id,
                vulnerable=cpe_match['vulnerable'],
                version_end_excluding=cpe_match['versionEndExcluding']
            )

            database_session.add(cpe_match)
            database_session.commit()
        except IntegrityError:
            database_session.rollback()
            pytest.fail("IntegrityError occurred while inserting CPE Match")



from arepo.base import Base

from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, PrimaryKeyConstraint, Boolean, Float


class DatasetModel(Base):
    __tablename__ = "dataset"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    description = Column('description', String, nullable=True)
    vulnerabilities = relationship("VulnerabilityModel", secondary="dataset_vulnerability")

    def __len__(self):
        return len(self.vulnerabilities)


class DatasetVulnerabilityModel(Base):
    __tablename__ = 'dataset_vulnerability'
    __table_args__ = (
        PrimaryKeyConstraint('dataset_id', 'vulnerability_id'),
    )

    dataset_id = Column('dataset_id', Integer, ForeignKey('dataset.id'))
    vulnerability_id = Column('vulnerability_id', String, ForeignKey('vulnerability.id'))


class ProfileModel(Base):
    __tablename__ = "profile"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    has_code = Column('has_code', Boolean, nullable=False)
    has_exploit = Column('has_exploit', Boolean, nullable=False)
    has_advisory = Column('has_advisory', Boolean, nullable=False)
    single_commit = Column('single_commit', Boolean, nullable=False)
    start_year = Column('start_year', Integer, nullable=False)
    end_year = Column('end_year', Integer, nullable=True)
    start_score = Column('start_score', Float, nullable=False)
    end_score = Column('end_score', Float, nullable=False)
    min_changes = Column('min_changes', Integer, nullable=False)
    max_changes = Column('max_changes', Integer, nullable=True)
    min_files = Column('min_files', Integer, nullable=False)
    max_files = Column('max_files', Integer, nullable=True)
    # TODO: make it a relationship to hold more extensions
    extension = Column('extension', String, nullable=True)
    # TODO: should include the size / count


class ProfileCWEModel(Base):
    __tablename__ = 'profile_cwe'
    __table_args__ = (
        PrimaryKeyConstraint('profile_id', 'cwe_id'),
    )

    profile_id = Column('profile_id', Integer, ForeignKey('profile.id'))
    cwe_id = Column('cwe_id', Integer, ForeignKey('cwe.id'))


class CompletionModel(Base):
    # TODO: move this model to a different place
    __tablename__ = "completion"

    id = Column('id', String, primary_key=True)
    object = Column('object', String, nullable=False)
    created = Column('created', Integer, nullable=False)
    model = Column('model', String, nullable=False)
    prompt = Column('prompt', String, nullable=False)
    completion = Column('completion', String, nullable=False)
    finish_reason = Column('finish_reason', String, nullable=False)
    prompt_tokens = Column('prompt_tokens', Integer, nullable=False)
    completion_tokens = Column('completion_tokens', Integer, nullable=False)
    total_tokens = Column('total_tokens', Integer, nullable=False)


class WeaknessModel(Base):
    # TODO: move this model to a different place
    __tablename__ = "weakness"

    id = Column('id', Integer, primary_key=True)
    tuple = Column('tuple', String, nullable=True)
    vulnerability_id = Column('vulnerability_id', String, ForeignKey('vulnerability.id'))
    # TODO: there should be a table to map the Weakness to the Completion table
    completion_id = Column('completion_id', String, ForeignKey('completion.id'))




import pytest
import hashlib

from arepo.models.source import SourceModel
from arepo.models.common.reference import ReferenceModel, ReferenceAssociationModel
from arepo.models.common.tag import TagModel, TagAssociationModel


reference_data = [
    {
      "url": "https://bsg.tech/blog/cve-2022-25854-stored-xss-in-yaireo-tagify-npm-module/",
      "source": "report@snyk.io",
      "tags": [
        "Exploit",
        "Patch",
        "Third Party Advisory"
      ]
    },
    {
      "url": "https://github.com/yairEO/tagify/commit/198c0451fad188390390395ccfc84ab371def4c7",
      "source": "report@snyk.io",
      "tags": [
        "Patch",
        "Third Party Advisory"
      ]
    },
    {
      "url": "https://github.com/yairEO/tagify/issues/988",
      "source": "report@snyk.io",
      "tags": [
        "Exploit",
        "Issue Tracking",
        "Third Party Advisory"
      ]
    },
    {
      "url": "https://github.com/yairEO/tagify/releases/tag/v4.9.8",
      "source": "report@snyk.io",
      "tags": [
        "Release Notes",
        "Third Party Advisory"
      ]
    },
    {
      "url": "https://snyk.io/vuln/SNYK-JS-YAIREOTAGIFY-2404358",
      "source": "report@snyk.io",
      "tags": [
        "Third Party Advisory"
      ]
    }
]


# change the format of the reference data to be a dict with ids
reference_data = {hashlib.md5(reference["url"].encode('utf-8')).hexdigest(): reference for reference in reference_data}


@pytest.mark.dependency(depends=["test_insert_sources"])
@pytest.mark.dependency(name="test_insert_references")
def test_insert_references(database_session):
    from sqlalchemy.exc import IntegrityError
    tag_ids = {tag.name: tag.id for tag in database_session.query(TagModel).all()}

    for _id, reference in reference_data.items():
        source = database_session.query(SourceModel).filter(SourceModel.email == reference["source"]).first()

        try:
            # Creating ReferenceModel object
            reference_model = ReferenceModel(
                id=_id,
                url=reference["url"]
            )

            database_session.add(reference_model)
            database_session.commit()

            reference_association = ReferenceAssociationModel(
                reference_id=_id,
                vulnerability_id="CVE-2022-25854",
                source_id=source.id
            )

            database_session.add(reference_association)
            database_session.commit()

            for tag in reference["tags"]:
                # Creating ReferenceTagModel object
                tag_association = TagAssociationModel(
                    reference_id=_id,
                    tag_id=tag_ids[tag],
                    source_id=source.id
                )

                database_session.add(tag_association)
                database_session.commit()
        except IntegrityError:
            database_session.rollback()
            pytest.fail("IntegrityError occurred while inserting references")


@pytest.mark.dependency(depends=["test_insert_references"])
def test_get_inserted_references(database_session):
    # Query the database to retrieve the inserted references
    retrieved_references = database_session.query(ReferenceModel).all()
    tag_ids = [tag.id for tag in database_session.query(TagModel).all()]

    # Check if the retrieved references exist
    assert retrieved_references is not None
    assert len(retrieved_references) == len(reference_data)

    for reference in retrieved_references:
        # Assert the retrieved data matches the expected values
        assert reference.url == reference_data[reference.id]["url"]
        associations = database_session.query(ReferenceAssociationModel).filter_by(reference_id=reference.id).all()

        assert associations is not None
        assert len(associations) == 1

        source_id = database_session.query(SourceModel).filter_by(email=reference_data[reference.id]["source"]).first().id

        for association in associations:
            assert association.source_id == source_id
            assert association.vulnerability_id == "CVE-2022-25854"

        # Query the database to retrieve the inserted reference tags
        tag_associations = database_session.query(TagAssociationModel).filter_by(reference_id=reference.id).all()

        # Check if the retrieved reference tags exist
        assert tag_associations is not None
        assert len(tag_associations) == len(reference_data[reference.id]["tags"])

        for reference_tag in tag_associations:
            # Assert the retrieved data matches the expected values
            assert reference_tag.tag_id in tag_ids



from arepo.models.common.platform.configuration import *
from arepo.models.common.platform.product import *
from arepo.models.common.platform.cpe import *
from arepo.models.common.platform.vendor import *



import pandas as pd
from pathlib import Path

from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, PrimaryKeyConstraint

from arepo.base import Base
from arepo.mixins import EntityLoaderMixin, AssociationLoaderMixin


class LabelModel(Base):
    __tablename__ = "label"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)


class FunctionModel(Base):
    __tablename__ = "function"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    commit_file_id = Column(String, ForeignKey('commit_file.id'))
    start_line = Column('start_line', Integer, nullable=False)
    start_col = Column('start_col', Integer, nullable=False)
    end_line = Column('end_line', Integer, nullable=False)
    end_col = Column('end_col', Integer, nullable=False)
    size = Column('size', Integer, nullable=False)
    content = Column('content', String, nullable=False)


# TODO: reconsider the purpose/existence of this model
class TopicModel(Base, EntityLoaderMixin):
    __tablename__ = "topic"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    repositories = relationship("RepositoryModel", secondary="repository_topic", backref='topics')

    @staticmethod
    def populate(tables_path: Path):
        topics_df = pd.read_csv(f'{tables_path}/topics.csv')
        Base.metadata.bind.execute(TopicModel.__table__.insert(), topics_df.to_dict(orient="records"))


# TODO: idem
class RepositoryTopicModel(Base, AssociationLoaderMixin):
    __tablename__ = 'repository_topic'
    __table_args__ = (
        PrimaryKeyConstraint('repository_id', 'topic_id'),
    )

    repository_id = Column('repository_id', String, ForeignKey('repository.id'))
    topic_id = Column('topic_id', String, ForeignKey('topic.id'))
